--[[
script version 2.1
this pricing script is designed for individuals to create or collect there own pricing information in whatever means they so choose and import that information into the magic album application.
to do so create the file "cardprices.txt" in the root folder of the magic album application.  inside the file each line represents a card and its associated information.  the expected line format is as follows:
(cardname{tab}cardprice{tab}setname{tab}cardlanguage{tab}cardversion{tab}foil/nonfoil{tab}cardnotes)
valid complete example line:
Abuna Acolyte	0.24	Scars of Mirrodin	English	*	nonfoil	user defined data
Abuna Acolyte	0.24	Scars of Mirrodin	English	0	nonfoil	site specific information

valid part-complete example line:
Abuna Acolyte	0.24	Scars of Mirrodin			

invalid line examples:
Abuna Acolyte	0.24	SOM	eng	0	nonfoil
Abuna Acolyte	0.24		eng	0	nonfoil

requirements:
setname must match setnames defined in "tab" table (see below)
cardlanguage must match a language  defined in the suplang table (see below)
price values must not contain commas
you must define a card name
you must define a card price
you must define a card set name
all other values if not present will use default values listed below

defaults:
cardlanguage will default to english (eng)
cardversion will default to * (all versions)
cardfoiling will default to nonfoil

variable descriptions:
nfnum 	non foil numbers	the number to start counting from when a new non foil card is imported (recommended to leave at 0)
fnum	foil numbers		the number to start counting from when a new foil card is imported (recommended to leave at 0)
cpf		card price file		file (including path if not local) to the file containing the card pricing information

tab table description:
id		id					setid used in the magic album application
setname	setname				name of set associated with id
block	blockname			name of block the setname is associated with

suplang table description:
full	language full name		MA supported language full name
abbr	language abbr name		MA supported language abbreviation letters
id		language id number		MA supported language identification number
--]]

tab ={
	{id = 790, setname = "duel decks: izzet vs. golgari", block = "Duel Decks"},
	{id = 789, setname = "from the vault: realms", block = "From the Vault"},
	{id = 788, setname = "magic 2013", block = "Core Sets"},
	{id = 787, setname = "planechase 2012 edition", block = "Casual Supplements"},
	{id = 786, setname = "avacyn restored", block = "Innistrad"},
	{id = 785, setname = "duel decks: venser vs. koth", block = "Duel Decks"},
	{id = 784, setname = "dark ascension", block = "Innistrad"},
	{id = 783, setname = "premium deck series: graveborn", block = "Decks"},
	{id = 782, setname = "innistrad", block = "Innistrad"},
	{id = 781, setname = "duel decks: ajani vs. nicol bolas", block = "Duel Decks"},
	{id = 780, setname = "from the vault: legends", block = "From the Vault"},
	{id = 779, setname = "magic 2012", block = "Core Sets"},
	{id = 778, setname = "magic: the gathering commander", block = "Casual Supplements"},
	{id = 778, setname = "magic: the gathering commander oversized", block = "Casual Supplements"},
	{id = 777, setname = "duel decks: knights vs. dragons", block = "Duel Decks"},
	{id = 776, setname = "new phyrexia", block = "Scars of Mirrodin"},
	{id = 775, setname = "mirrodin besieged", block = "Scars of Mirrodin"},
	{id = 774, setname = "premium deck series: fire & lightning", block = "Decks"},
	{id = 773, setname = "scars of mirrodin", block = "Scars of Mirriodin"},
	{id = 772, setname = "duel decks: elspeth vs. tezzeret", block = "Duel Decks"},
	{id = 771, setname = "from the vault: relics", block = "From the Vault"},
	{id = 770, setname = "magic 2011", block = "Core Sets"},
	{id = 769, setname = "archenemy", block = "Casual Supplements"},
	{id = 768, setname = "duels of the planeswalkers", block = "Decks"},
	{id = 767, setname = "rise of the eldrazi", block = "Zendikar"},
	{id = 766, setname = "duel decks: phyrexia vs. the coalition", block = "Duel Decks"},
	{id = 765, setname = "worldWake", block = "Zendikar"},
	{id = 764, setname = "premium deck series: slivers", block = "Decks"},
	{id = 763, setname = "duel decks: garruk vs. liliana", block = "Duel Decks"},
	{id = 762, setname = "zendikar", block = "Zendikar"},
	{id = 761, setname = "planechase", block = "Casual Supplements"},
	{id = 760, setname = "from the vault: exiled", block = "From the Vault"},
	{id = 759, setname = "magic 2010", block = "Core Sets"},
	{id = 758, setname = "alara reborn", block = "Shards of Alara"},
	{id = 757, setname = "duel decks: divine vs. demonic", block = "Duel Decks"},
	{id = 756, setname = "conflux", block = "Shards of Alara"},
	{id = 755, setname = "duel decks: jace vs. chandra", block = "Duel Decks"},
	{id = 754, setname = "shards of alara", block = "Shards of Alara"},
	{id = 753, setname = "from the vault: dragons", block = "From the Vault"},
	{id = 752, setname = "eventide", block = "Lorwyn-Shadowmoor"},
	{id = 751, setname = "shadowmoor", block = "Lorwyn-Shadowmoor"},
	{id = 750, setname = "morningtide", block = "Lorwyn-Shadowmoor"},
	{id = 740, setname = "duel decks: elves vs. goblins", block = "Duel Decks"},
	{id = 730, setname = "lorwyn", block = "Lorwyn-Shadownmoor"},
	{id = 720, setname = "tenth edition", block = "Core Sets"},
	{id = 710, setname = "future sight", block = "Time Spiral"},
	{id = 700, setname = "planar chaos", block = "Time Spiral"},
	{id = 690, setname = "time spiral timeshifted", block = "Time Spiral"},
	{id = 680, setname = "time spiral", block = "Time Spiral"},
	{id = 675, setname = "coldsnap theme decks", block = "Ice Age"},
	{id = 670, setname = "coldsnap", block = "Ice Age"},
	{id = 660, setname = "eissension", block = "Ravnica"},
	{id = 650, setname = "guildpact", block = "Ravnica"},
	{id = 640, setname = "ravnica: city of guilds", block = "Ravnica"},
	{id = 635, setname = "magic encyclopedia", block = ""},
	{id = 630, setname = "9th edition", block = "Core Sets"},
	{id = 620, setname = "saviors of kamigawa", block = "Kamigawa"},
	{id = 610, setname = "betrayers of kamigawa", block = "Kamigawa"},
	{id = 600, setname = "unhinged", block = "Un Sets"},
	{id = 590, setname = "champions of kamigawa", block = "Kamigawa"},
	{id = 580, setname = "fifth dawn", block = "Mirrodin"},
	{id = 570, setname = "darksteel", block = "Mirrodin"},
	{id = 560, setname = "mirrodin", block = "Mirrodin"},
	{id = 550, setname = "8th edition", block = "Core Sets"},
	{id = 540, setname = "scourge", block = "Onslaught"},
	{id = 530, setname = "legions", block = "Onslaught"},
	{id = 520, setname = "onslaught", block = "Onslaught"},
	{id = 510, setname = "judgment", block = "Odyssey"},
	{id = 500, setname = "torment", block = "Odyssey"},
	{id = 490, setname = "deckmasters", block = "Box Sets"},
	{id = 480, setname = "odyssey", block = "Odyssey"},
	{id = 470, setname = "apocalypse", block = "Invasion"},
	{id = 460, setname = "7th edition", block = "Core Sets"},
	{id = 450, setname = "planeshift", block = "Invasion"},
	{id = 440, setname = "beatdown", block = "Box Sets"},
	{id = 430, setname = "invasion", block = "Invasion"},
	{id = 420, setname = "prophecy", block = "Masques"},
	{id = 415, setname = "starter 2000", block = "Portal / Starter"},
	{id = 410, setname = "nemesis", block = "Masques"},
	{id = 405, setname = "battle royale", block = "Box Sets"},
	{id = 400, setname = "mercadian masques", block = "Masques"},
	{id = 390, setname = "starter 1999", block = "Portal / Starter"},
	{id = 380, setname = "portal three kingdoms", block = "Portal / Starter"},
	{id = 370, setname = "urza's destiny", block = "Urza"},
	{id = 360, setname = "6th edition", block = "Core Sets"},
	{id = 350, setname = "urza's legacy", block = "Urza"},
	{id = 340, setname = "anthologies", block = "Reprint"},
	{id = 330, setname = "urza's saga", block = "Urza"},
	{id = 320, setname = "unglued", block = "Un Sets"},
	{id = 310, setname = "portal second age", block = "Portal / Starter"},
	{id = 300, setname = "exodus", block = "Tempest"},
	{id = 290, setname = "stronghold", block = "Tempest"},
	{id = 280, setname = "tempest", block = "Tempest"},
	{id = 270, setname = "weatherlight", block = "Mirage"},
	{id = 260, setname = "portal", block = "Portal / Starter"},
	{id = 250, setname = "5th edition", block = "Core Sets"},
	{id = 240, setname = "visions", block = "Mirage"},
	{id = 230, setname = "mirage", block = "Mirage"},
	{id = 225, setname = "introductory two-player set", block = ""},
	{id = 220, setname = "alliances", block = "Ice Age"},
	{id = 210, setname = "homelands", block = "Ice Age"},
	{id = 200, setname = "chronicles", block = "Reprint"},
	{id = 190, setname = "ice age", block = "Ice Age"},
	{id = 180, setname = "4th edition", block = "Core Sets"},
	{id = 170, setname = "fallen empires", block = "Stand Alone"},
	{id = 160, setname = "the dark", block = "Stand Alone"},
	{id = 150, setname = "legends", block = "Stand Alone"},
	{id = 140, setname = "revised", block = "Core Sets"},
	{id = 130, setname = "antiquities", block = "Stand Alone"},
	{id = 120, setname = "arabian nights", block = "Stand Alone"},
	{id = 110, setname = "unlimited", block = "Core Sets"},
	{id = 100, setname = "beta", block = "Core Sets"},
	{id = 90, setname = "alpha", block = "Core Sets"},
	{id = 70, setname = "vanguard", block = "Casual Supplements"},
	{id = 50, setname = "full box promotion", block = "Promo"},
	{id = 45, setname = "magic premiere shop", block = "Promo"},
	{id = 43, setname = "two-headed giant promos", block = "Promo"},
	{id = 42, setname = "summer of magic promos", block = "Promo"},
	{id = 41, setname = "happy holidays promos", block = "Promo"},
	{id = 40, setname = "arena league promos", block = "Promo"},
	{id = 33, setname = "world championship cards", block = "Decks"},
	{id = 32, setname = "pro tour promos", block = "Promo"},
	{id = 31, setname = "grand prix promos", block = "Promo"},
	{id = 30, setname = "friday night magic promos", block = "Promo"},
	{id = 27, setname = "alternate art lands", block = "Promo"},
	{id = 26, setname = "magic game day", block = "Promo"},
	{id = 25, setname = "judge gift cards", block = "Promo"},
	{id = 24, setname = "champs promos", block = "Promo"},
	{id = 23, setname = "gateway & wpn promos", block = "Promo"},
	{id = 22, setname = "prerelease promos", block = "Promo"},
	{id = 21, setname = "release & launch parties promos", block = "Promo"},
	{id = 20, setname = "magic player rewards", block = "Promo"},
	{id = 15, setname = "convention promos", block = "Promo"},
	{id = 10, setname = "junior series promos", block = "Promo"},
	{id = 9, setname = "video game promos", block = "Promo"},
	{id = 8, setname = "stores promos", block = "Promo"},
	{id = 7, setname = "magazine inserts", block = "Promo"},
	{id = 6, setname = "comic inserts", block = "Promo"},
	{id = 5, setname = "book inserts", block = "Promo"},
	{id = 4, setname = "richard garfield events", block = "Promo"},
	{id = 2, setname = "dci legend membership", block = "Promo"},
	{id = 1, setname = "other promos", block = "Promo"},
	}
	
suplang = {
	  {full = "english", abbr="ENG", id=1},
	  {full = "russian", abbr="RUS", id=2},
	  {full = "german", abbr="GER", id=3},
	  {full = "french", abbr="FRA", id=4},
	  {full = "italian", abbr="ITA", id=5},
	  {full = "portuguese", abbr="POR", id=6},
	  {full = "spanish", abbr="SPA", id=7},
	  {full = "japanese", abbr="JPN", id=8},
	  {full = "simplified chinese", abbr="ZHC", id=9},
	  {full = "traditional chinese", abbr="ZHT", id=10},
	  {full = "korean", abbr="KOR", id=11},
	  {full = "hebrew", abbr="HEB", id=12},
	  {full = "arabic", abbr="ARA", id=13},
	  {full = "latin", abbr="LAT",id =14},
	  {full = "sanskrit", abbr="SAN", id=15},
	  {full = "ancient greek", abbr="GRC", id=16},
}
	
function explode(d,p)
	local t, ll
	t={}
	ll=0
	if(#p == 1) then return p end
	while true do
		l=string.find(p,d,ll+1,true)
		if l~=nil then
			table.insert(t, string.sub(p,ll,l-1))
			ll=l+1
		else
			table.insert(t, string.sub(p,ll))
			break
		end
	end
	return t
end
	
function trimspaces (s)
      return (string.gsub(s, "^%s*(.-)%s*$", "%1"))
end

--declare user defined variable(s)
nfnum = 0					--recommended value: 0
fnum = 0					--recommended value: 0
cpf = "cardprices.txt"		--recommended value: name according to website the prices are pulled from

function ImportPrice(importfoil, importlangs, importsets)
	local alllangs = nil
	local allsets = nil
	local totalsetcount = 0
	
	-- identify user defined types of foiling to import
	if string.lower(importfoil) == "y" then ma.Log("Importing Non-Foil (+) Foil Card Prices") end
	if string.lower(importfoil) == "o" then ma.Log("Importing Foil Only Card Prices") end
	if string.lower(importfoil) == "n" then ma.Log("Importing Non-Foil Only Card Prices") end
	
	-- identify user defined sets to import
	for sid,sname in pairs(importsets) do
		if allsets == nil then
			allsets = sname
		else
			allsets = allsets .. "," .. sname
		end
		totalsetcount = totalsetcount + 1
	end
	ma.Log("Importing Sets: [" .. allsets .. "]")
	
	-- identify user defined languages to import
	for lid,lname in pairs(importlangs) do
		if alllangs == nil then
			alllangs = lname
		else
			alllangs = alllangs .. "," .. lname
		end
	end
	ma.Log("Importing Languages: [" .. alllangs .. "]")
	fc = ma.GetFile(cpf)
	if fc ~= nil then
		--expected line format (cardname{tab}cardprice{tab}setname{tab}cardlanguage{tab}cardversion{tab}foil/nonfoil{tab}cardnotes)
		fls = explode("\n",fc)
		for i in ipairs(fls) do
			line = fls[i]
			linesplit = explode("\t",line)
			lsc = 1
			
			--clear any previous card details
			local cardname = nil
			local cardcost = nil
			local cardset = nil
			local cardlanguage = nil
			local cardversion = nil
			local cardfoiling = nil
			local cardsetid = nil
			local cardblockname = nil
			local cardabbr = nil
			local cardlangid = nil
			local cardnotes = nil
			local els = 7
			local cardlinevalid = "no"
			local currentsetcount = 0
			
			--define new card details
			for j in ipairs(linesplit) do
				if lsc == 1 then cardname = trimspaces(linesplit[j]) end
				if lsc == 2 then cardcost = trimspaces(linesplit[j]) end
				if lsc == 3 then cardset = trimspaces(linesplit[j]) end
				if lsc == 4 then cardlanguage = linesplit[j] end
				if lsc == 5 then cardversion = linesplit[j] end
				if lsc == 6 then cardfoiling = linesplit[j] end
				if lsc == 7 then cardnotes = linesplit[j] end
				if lsc > els then break end 
				lsc = lsc + 1
			end

			--identify card set id  and block from setname
			for _, rec in ipairs(tab) do
				if string.lower(cardset) == rec.setname then
					cardsetid = rec.id
					cardblockname = rec.block
				end 
			end
			--identify card language abbr
			for _, rec in ipairs(suplang) do
				if string.lower(cardlanguage) == rec.full then
					cardabbr = rec.abbr
					cardlangid = rec.id
				end
			end
		
			--verify required fields are set before attempting to set card price
			if lsc == 8 then
				if cardsetid ~= nil then
					if cardname ~= nil then
						if cardcost ~= nil then
							cardlinevalid = "yes"
							--define default values if values are not present
							if cardversion == "0" then cardversion = "" end
							if cardversion == "1" then cardversion = 1 end
							if cardversion == "2" then cardversion = 2 end
							if cardversion == "3" then cardversion = 3 end
							if cardversion == "4" then cardversion = 4 end
							if cardversion == nil then cardversion = "*" end
							if cardversion == "" then cardversion = "*" end
							if cardlangid == nil then cardlanguage = 1 end
							if cardlangid == "" then cardlanguage = 1 end
							if cardfoiling == nil then cardfoiling = "nonfoil" end
							if cardfoiling == "" then cardfoiling = "nonfoil" end
							
							for sid,sname in pairs(importsets) do
								if string.lower(sname) == string.lower(cardset) then
									currentsetcount = currentsetcount + 1
									ma.SetProgress(cardname .. " [" .. sname .. "]", 100*currentsetcount/totalsetcount)
									for lid,lname in pairs(importlangs) do
										if string.lower(lname) == string.lower(cardlanguage) then
											if string.lower(importfoil) ~= "o" then
												if string.lower(cardfoiling) == "nonfoil" then
													--set nonfoil card price
													priceresult = ma.SetPrice(cardsetid,cardlangid,cardname,cardversion,cardcost,0)
													nfnum = nfnum + 1
												end
											end
										
											if string.lower(importfoil) ~= "n" then
												if string.lower(cardfoiling) == "foil" then
													--set foil card price
													priceresult = ma.SetPrice(cardsetid,cardlangid,cardname,cardversion,0,cardcost)
													fnum = fnum + 1
												end
											end
										end
									end
								end
							end
						end
					end
				end
			end
			
			--write invalid formated lines to MA log file
			if cardlinevalid == "no" then
				ma.Log("[WARNING]\tInvalidPriceLine: (" .. line .. ")")
				ma.Log(debugline)
			end
		end
		ma.Log(nfnum .. " total non-foil card price(s) set")
		ma.Log(fnum .. " total foil card price(s) set")
	end
end
